#!/usr/bin/env python3

import os
import glob 
from pathlib import Path
import pandas as pd



def get_pool_dicts(exp_design):
    
    """
    Get file2pool_dict indicating which input file belongs to which pool and
    pool2dim dict indicating which  pool belongs to which dimension
        


    Parameters
    ----------
    exp_design : str
       Name of file encoding experimental design. The experimental design file 
       should have columns, Filename, Poolname and Pooldimension.

    Returns
    -------
    file2pool_dict: dict
        indicating which input file belongs to which pool
    pool2dim_dict: dict
        indicating which  pool belongs to which dimension

    """
    
    exp_design_ = pd.read_csv(exp_design, index_col=0)

    file2pool_dict = {Path(i).stem.split(".")[0]:exp_design_.loc[i,"Poolname"] for i in exp_design_.index.unique()}
    
    exp_design_ = pd.read_csv(exp_design, index_col=2)

    pool2dim_dict = {i:exp_design_.loc[i,"Poolname"].unique().tolist() for i in exp_design_.index.unique()}

    return file2pool_dict, pool2dim_dict
    
def get_input_files(experiment):
    """
    
    
    Parameters
    ----------
    experiment : LibraryExperiment
        Object holding input parameters.

    Returns
    -------
    paths : list of strings
        paths of input files
    names : list of strings
        names of input files

    """    

    exp_design = pd.read_csv(experiment.exp_design)
    assert all(exp_design.columns == ["Filename", "Poolname", "Pooldimension"]), \
        "Please provide an experimental design file with columns: Filename, Poolname, Pooldimension"
    names = [Path(i).stem.split(".")[0] for i in exp_design["Filename"]]
    paths = [os.path.join(experiment.input_dir, i) for i in exp_design["Filename"]]
    #assert len(paths) != 0, "No input files found in " + str(experiment.input_dir)

    return paths, names




def set_up_tmpdir():
    """´
    Set up temp directory and clean up previous results.

    
    Returns
    -------
    None.

    """
    
    
    if not os.path.exists("temp"):
        os.makedirs("temp")
    




import numpy as np
import pandas as pd
import string
from itertools import product
from sklearn.linear_model import Lasso, OrthogonalMatchingPursuit
import multiprocessing as mp
from itertools import repeat

def getX(dims, pools):
    """
    Generate design matrix of combinatorial pooling. Columns corresponds to 
    wells on the grid and the rows to the pools. Entries with 1 indicate that 
    a well is part of a pool. 

    Parameters
    ----------
    dims :  dict
        indicating which  pool belongs to which dimension
    pools : list of str
        holding names of pools
    Returns
    -------
    X : pandas dataframe
        Pooling design matrix

    """
    # build design matrix
    colnames = []
    
    for i in product(*dims.values()):
        colnames.append("_".join(i))
    X = pd.DataFrame(0, index=pools, columns = colnames)
    for i in colnames:
        idx = i.split("_")
        X.loc[idx,i] = 1
    return X



def fit(mut, X, dims):
    """
    Fit regularized linear model for a given mutant given the pooling design
    matrix X and return predicted and possible well locations

    Parameters
    ----------
    mut : numpy array
        row of the mutant count matrix, holding the read counts for all pools 
        for a given mutant
    X : pandas dataframe
        Pooling design matrix
    dims : dict
        indicating which  pool belongs to which dimension

    Returns
    -------
    pred: list of str
        predicted wells
    possible: list of str
        possible wells

    """
    nonzero = []
    for j in dims:
        nonzero.append(np.intersect1d(mut.index[mut != 0] , dims[j]))
    
    red = max([len(i) for i in nonzero])
    mut_ = mut[mut!=0]
    omp = OrthogonalMatchingPursuit(n_nonzero_coefs=red, fit_intercept=False)
    possible = ["_".join(i) for i in product(*nonzero)]
    X_  = X.loc[mut_.index , possible]
    omp = omp.fit(X_, mut_)
    pred = X_.columns[omp.coef_ !=0].tolist()
    return pred, possible


def predict_ambiguous_locations(data, experiment):
    """
    Wrapper to run predictions for ambiguous mutants.

    Parameters
    ----------
    data : pandas dataframe
        count matrix of ambiguous mutants
    experiment : LibraryExperiment
        Object holding input parameters.

    Returns
    -------
    locations : pandas dataframe
        holding the predicted locations of mutants and properties 
        (genomic coordinates, tn orientation, ambiguity etc.)

    """
    dims = experiment.pool_dims
    pools = experiment.pools
    pool_reads = data[pools].astype(int)
    X = getX(dims, pools)
    
    pred = []
    possible = []
    for i in range(pool_reads.shape[0]):
        res = fit(pool_reads.iloc[i], X, dims)
        pred.append(res[0])
        possible.append(res[1])
    
    pred = np.array([";".join(i) for i in pred])
    possible = np.array([";".join(i) for i in possible])
    amb = np.array(["Ambiguous" for i in range(data.shape[0])])
    
    if "Barcode" in data.columns:
        outcols = ["Feature", "Orientation", "Barcode",
                  "Predicted_wells", "Possible_wells", "Ambiguity"]
        locations = np.vstack((data["Feature"], data["Orientation"], data["Barcode"], 
                  pred, possible, amb)).T
        locations = pd.DataFrame(locations, columns =outcols)
    else:
        outcols = ["Feature", "Orientation",
                  "Predicted_wells", "Possible_wells", "Ambiguity"]
        locations = np.vstack((data["Feature"], data["Orientation"], 
                  pred, possible, amb)).T
        locations = pd.DataFrame(locations, columns =outcols)
    return locations



from pysudoku_package.libraryexperiment import LibraryExperiment
import sys
import click
import pandas as pd


@click.command()
@click.argument("input_dir", type=str)
@click.argument("exp_design", type=str)
@click.option("--cores", '-c',
              metavar='<str>',
              type=int,
              default=1,
              help='Number of cpu cores to use')
@click.option("--map_quality", '-mq',
              metavar='<int>',
              type=int,
              default=30,
              help='Minimum bowtie2 alignment quality score to include read')
@click.option("--seq_quality", '-sq',
              metavar='<int>',
              type=int,
              default=10,
              help='Minimum phred score for each base to include read')
@click.option("--gb_ref", '-gb',
              metavar='<str>',
              type=str,
              help='Path to genbank reference file')
@click.option("--bowtie_ref", '-br',
            metavar='<str>',
            type=str,
            help='Path to bowtie reference files')
@click.option("--tn_seq", '-t',
              metavar='<str>',
              type=str,
              help='Transposon sequence')
@click.option("--tn_mismatch", '-tm',
              metavar='<int>',
              type=int,
              default=1,
              help='Number of transposon mismatches allowed')
@click.option("--use_barcodes", '-b',
              type=bool,
              metavar='<bool>',
              default=True,
              help='Whether to use barcodes for deconvolution')
@click.option("--bar_upstream", '-bu',
              metavar='<str>',
              type=str,
              help='Upstream sequence of barcode')
@click.option("--bar_downstream", '-bd',
              metavar='<str>',
              type=str,
              help='Downstream sequence of barcode')
@click.option("--filter_thr", '-thr',
              metavar='<int>',
              type=float,
              default=0.05,
              help='Count filter threshold')
def run(input_dir, exp_design, cores,map_quality,seq_quality,gb_ref, bowtie_ref, 
                                tn_seq, tn_mismatch, 
                                use_barcodes, bar_upstream, bar_downstream, filter_thr):
    """ 
    Run pysudoku to process sequencing data from combnatorially pooled 
    arrayed libraries and infer the most likely locations for each mutant.
    INPUT_DIR should be a path to the directory containing the input fastq
    files and EXP_DESIGN a csv file with columns Filename, Poolname and 
    Pooldimension indicating which files belong to which pool and pooling 
    dimension.
    """
    if tn_seq == None:
        raise ValueError("Please provide the transposon sequence!")
    if use_barcodes == True:
        if bar_upstream == None:
            raise ValueError("Please provide the sequence upstream to the transposon! If you don't want to consider Barcodes please set -b to False!")
        if bar_downstream == None:
            raise ValueError("Please provide the sequence downstream to the transposon! If you don't want to consider Barcodes please set -b to False!")
            
    
    experiment = LibraryExperiment(cores,map_quality,seq_quality,gb_ref, bowtie_ref, 
                                    tn_seq, tn_mismatch, input_dir, exp_design,
                                    use_barcodes, bar_upstream, bar_downstream, filter_thr)
    experiment.pools
    print("Found", str(len(experiment.file2pool)), "input files!")
    print("Running pysudoku with", str(len(experiment.pools)), "pools and ",
          str(len(experiment.pool_dims)), "pooling dimensions!")
    print("Finding transposon sequence and trimming reads to genomic sequence!")
    experiment.get_genomic_seq()    
    print("Trimming done!")
    print("Starting alignment to reference!")    
    experiment.align_genomic_seq()       
    print("Alignment done!")    
    experiment.write_count_matrix()
    print("Inferring most likely locations and writing location summary!")
    experiment.deconvolve()
    print("Done!")


@click.command()
@click.argument("count_matrix", type=str)
@click.argument("exp_design", type=str)
@click.option("--cores", '-c',
              metavar='<str>',
              type=int,
              default=1,
              help='Number of cpu cores to use')
@click.option("--gb_ref", '-gb',
              metavar='<str>',
              type=str,
              help='Path to genbank reference file')
@click.option("--filter_thr", '-thr',
              metavar='<int>',
              type=float,
              default=0.05,
              help='Count filter threshold')
def deconvolve(count_matrix, exp_design, cores, gb_ref, filter_thr) :
    """ 
    Run pysudoku to infer the most likely locations for each mutant from a 
    precomputed count matrix.
    COUNT_MATRIX should be a path to a precomputed count matrix and EXP_DESIGN 
    a csv file with columns Filename, Poolname and 
    Pooldimension indicating which files belong to which pool and pooling 
    dimension.
    """
    if count_matrix == None:
        raise ValueError("Please provide path to count matrix!")
   
    experiment = LibraryExperiment(cores,map_quality =None,seq_quality=None,gb_ref=gb_ref, bowtie_ref=None, 
                                    tn_seq=None, tn_mismatches=None, input_dir=None, exp_design=exp_design,
                                    use_barcodes=True, bar_upstream=None, bar_downstream=None, filter_thr=filter_thr)
    c = pd.read_csv(count_matrix)
    experiment.count_mat = c
    print("Inferring most likely locations and writing location summary!")
    experiment.deconvolve()
    print("Done!")



#!/usr/bin/env python3

import os
import numpy as np
from numba import njit
from pathlib import Path
import gzip 

from pysudoku_package.io import txt_writer


@njit
def binary_subtract(array1,array2,mismatch):
    
    """ Used for matching 2 sequences based on the allowed mismatches.
    Requires the sequences to be in numerical form"""
    
    miss=0
    for arr1,arr2 in zip(array1,array2):
        if arr1-arr2 != 0:
            miss += 1
        if miss>mismatch:
            return 0
    return 1

@njit
def border_finder(seq,read,mismatch): 
    
    """ Matches 2 sequences (after converting to int8 format)
    based on the allowed mismatches. Used for sequencing searching
    a start/end place in a read"""
    
    s=seq.size
    r=read.size
    fall_over_index = r-s-1
    for i,bp in enumerate(read): #range doesnt exist in njit
        comparison = read[i:s+i]
        finder = binary_subtract(seq,comparison,mismatch)
        if i > fall_over_index:
            return
        if finder != 0:
            return i

    
def seq2bin(sequence):
    
    """ Converts a string to binary, and then to 
    a numpy array in int8 format"""
    
    byte_list = bytearray(sequence,'utf8')
    return np.array((byte_list), dtype=np.int8)

def barcode_extractor(border_up_bin,border_down_bin,read_bin):
    match_up=border_finder(border_up_bin,read_bin,1)
    match_down=border_finder(border_down_bin,read_bin,1)
    return match_up,match_down
    

def read_data(file, experiment):
    
    """ reads the fastq files. Searches for transposon border, and returns
    the subsequent trimmed sequence, read ID, and pool ID"""

    tn_seq_bin = seq2bin(experiment.tn_seq)
    
    if experiment.use_barcodes:
        border_up_bin = seq2bin(experiment.bar_upstream)
        border_down_bin = seq2bin(experiment.bar_downstream)
    
    reading,trimmed_seq = [],[]
    count = 0

    filename = Path(file).stem.split(".")[0]
    poolname = experiment.file2pool[filename]
    quality_list = '!"#$%&' + "'()*+,-/0123456789:;<=>?@ABCDEFGHI"
    quality_set = set(quality_list[:int(experiment.seq_quality)-1])
    
    path_trimmed = os.path.join("temp", filename + "_trimmed_seq.fastq")
    with open(path_trimmed, "w") as output: # create empty file
        pass
    
    _, ext = os.path.splitext(file)
    if ext == ".gz":
        f = gzip.open(file, "rt") 
    else:
        f = open(file, "r") 
    for line in f:
        reading.append(line[:-1])

        if len(reading) == 4: #a read always has 4 lines
            count+=1
            ID = f"{count}:{poolname}:"
            read = reading[1]
            quality = reading[3]
            reading = []

            read_bin = seq2bin(read)
            match=border_finder(tn_seq_bin,read_bin,experiment.tn_mismatches)
            
            if match is not None:
                match+=len(tn_seq_bin)
                seq = read[match:]
                qual = quality[match:]
                # check if all bases have higher quality than param["phred"]
                if (len(quality_set.intersection(qual)) == 0):
                
                    if experiment.use_barcodes:
                        match_up,match_down=barcode_extractor(border_up_bin,border_down_bin,read_bin)
                        if (match_up is not None) & (match_down is not None):
                            barcode = read[match_up+len(border_up_bin):match_down]
                            ID = ID + f"{barcode}"
                        
                    trimmed_seq.append(f"@{ID}\n{seq}\n+\n{qual}\n")        
            
            #if psutil.virtual_memory().percent>=80:
            if len(trimmed_seq)>250000:
                txt_writer(path_trimmed, trimmed_seq)
                trimmed_seq = []
    f.close()
    txt_writer(path_trimmed, trimmed_seq)
    
    return path_trimmed





#!/usr/bin/env python3

import os



def writing_arg_decider(path):
    """
    whether to create or append to file.

    """
    arg = "w"
    if os.path.exists(path):
        arg = "a+"
    return arg

def txt_writer(path, outfile):
    
    """ writes the indicated outfile into a .txt file in the directory"""
    
    arg = writing_arg_decider(path)

    with open(path, arg) as output: #writes the output
        for line in outfile:
            output.write(line)
        
def file_compiler(outputed,inputed):
    
    """ compiles several files into one, deleting the temp files"""
    
    with open(outputed,"w") as out:
        for path in inputed:
            with open(path) as current:
                for line in current:
                    out.write(line)
            os.remove(path)
        out.close()
        
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pysudoku_package.config import get_input_files, get_pool_dicts, set_up_tmpdir
from pysudoku_package.io import txt_writer, file_compiler
from pysudoku_package.alignment import run_bowtie2, parse_bowtie2_output
from pysudoku_package.trimming import read_data
from pysudoku_package.generate_count_matrix import  get_count_matrix, get_hash_ids, filter_barcodes, filter_counts
from pysudoku_package.deconvolution import get_ambiguity, get_unambiguous_locations, coords2genes, transpose_location_summary
from pysudoku_package.predict_locations import predict_ambiguous_locations
import multiprocessing as mp
from itertools import repeat
import os
import pandas as pd
import numpy as np

class LibraryExperiment(object):

    
    def __init__(self, 
                 cores, 
                 map_quality, 
                 seq_quality, 
                 gb_ref, 
                 bowtie_ref, 
                 tn_seq,
                 tn_mismatches,
                 input_dir, 
                 exp_design,
                 use_barcodes, 
                 bar_upstream,
                 bar_downstream,
                 filter_thr):
        """
        

        Parameters
        ----------
        cores : int
            number of cores to use
        map_quality : int
            minimum bowtie2 alignment quality score for each base to include read
        seq_quality : int
            minimum phred score for each base to include read
        gb_ref : str
            path to genbank reference file
        bowtie_ref : str
            path to bowtie reference file
        tn_seq : str
            transposon sequence (e.g. ATTGCCTA)
        tn_mismatches : int
            number of transposon mismatches allowed
        input_dir : str
            path to directory holding the input fastq files
        exp_design : str
            path to file indicating experimental design. The experimental design file 
            should have columns, Filename, Poolname and Pooldimension
        use_barcodes : bool
            whether to include barcodes
        bar_upstream : str
            upstream sequence of barcode
        bar_downstream : str
            downstream sequence of barcode
        filter_thr : float
            threshold for local filter



        """
        # store input parameters
        self.cores = cores
        self.map_quality = map_quality
        self.seq_quality = seq_quality
        self.gb_ref = gb_ref
        self.bowtie_ref = bowtie_ref
        self.tn_seq = tn_seq
        self.tn_mismatches = tn_mismatches
        self.input_dir = input_dir
        self.exp_design = exp_design
        self.use_barcodes = use_barcodes
        self.filter_thr = filter_thr
        if use_barcodes:
            self.bar_upstream = bar_upstream
            self.bar_downstream = bar_downstream
        
        # hardcode paths for intermediate results
        self.bowtie_res = os.path.join("temp", "alignment_result.csv")
        self.alignment = os.path.join("temp", "alignment.sam")
        
        
        self.file2pool, self.pool_dims = get_pool_dicts(self.exp_design)
        self.pools = np.sort(np.unique([i for j in list(self.pool_dims.values()) for i in j])).tolist()
        



    def get_genomic_seq(self):
        
        """ 
        Trims all the reads based on the presence of the transposon recognizing
        sequence. Only keeps the downstream genomic sequences after the 
        transposon border site.
        
        Trimmed genomic sequences are written to temp/trimmed_sequences.fastq
        
        """
        
        self.input_file_paths, self.input_file_names = get_input_files(self)

        set_up_tmpdir()

        pool = mp.Pool(self.cores)

        paths = pool.starmap_async(read_data, zip(self.input_file_paths, repeat(self))).get()

        pool.close()
        pool.join()
        
        trimmed_paths = [item for item in paths]

        file_compiler(os.path.join("temp", "trimmed_sequences.fastq"), trimmed_paths)

    
    def align_genomic_seq(self):
        """
        Aligns trimmed reads to reference using bowtie2. The output of bowtie2 
        is parsed and stored in alignment_result.csv.
        """
        set_up_tmpdir()

        # clean previous alignments
        align_res = os.path.join("temp", "alignment_result.csv")
        if os.path.exists(align_res):
            os.remove(align_res)
            
        run_bowtie2(self)
        parse_bowtie2_output(self)
    
    def write_count_matrix(self):
        """
        Assembles count matrix from bowtie2 alignment. Filters out spurious 
        barcodes with very little read counts for a given coordinate.

        """
        bowtie_res = pd.read_csv(self.bowtie_res, 
                               names=["id_in_pool","pool","coord","orientation","barcode"],
                               dtype= {"id_in_pool": str, "pool": str, "coord":str, "orientation":str, "barcode":str})
            
        bowtie_res["barcode"] = bowtie_res["barcode"].astype(str)
    
        # barcodes that were not found are treated the same (can't distinguish between them)
        # concat orientation, coordinate and barcode to create unique ids
        bowtie_res["unique_id"] = bowtie_res["orientation"].astype(str) + \
            bowtie_res["coord"].astype(str)   + bowtie_res["barcode"].astype(str)
        
        encoded_ids = get_hash_ids(bowtie_res["unique_id"])
        
        assert len(np.unique(encoded_ids)) == len(np.unique(bowtie_res["unique_id"])), "Hashed ids not unique!"

        count_mat = get_count_matrix(encoded_ids, bowtie_res, self)
        count_mat.to_csv("count_matrix.csv", index=False, float_format="%.0f")

        filtered_count_mat = filter_barcodes(count_mat, self)
        filtered_count_mat.to_csv("filtered_count_matrix.csv", index=False, float_format="%.0f")
        
        self.count_mat = filtered_count_mat

        
    def deconvolve(self):
        """
        Deconvolve mutant count matrix and return summary output with 
        genes names.
        
        """
        # normalize to library size and filter low reads
        filtered_count_mat = self.count_mat
        counts = filtered_count_mat[self.pools].values.astype(float)
        counts = (counts/np.sum(counts, axis=0))*1e6
        counts = filter_counts(counts, thr=self.filter_thr)
        filtered_count_mat[self.pools] = counts
        self.count_mat = filtered_count_mat
        
        
        unambiguous_data, ambiguous_data= get_ambiguity(self)
        unambiguous_locations =  get_unambiguous_locations(unambiguous_data, self)
        ambiguous_locations = predict_ambiguous_locations(ambiguous_data, self)
        locations = pd.concat([unambiguous_locations, ambiguous_locations])
        
        # bad assertions/ count mat is longer than locations as it contains unpredictable mutants
        # assert np.all(locations["Feature"].values == self.count_mat["Feature"].values), \
        #     "Coordinates of locations and data not aligned!"
            
        # assert np.all(locations["Orientation"].values == self.count_mat["Orientation"].values), \
        #     "Orientations of locations and data not aligned!"
            
        # if "Barcode" in self.count_mat.columns:
        #     assert np.all(locations["Barcode"].values == self.count_mat["Barcode"].values), \
        #         "Barcodes of locations and data not aligned!"
        
        location_summary = coords2genes(self, locations)
        self.location_summary = location_summary
        location_summary.to_csv("mutant_location_summary.csv", index=False)
        
        transposed_location_summary = transpose_location_summary(self, location_summary)
        self.transposed_location_summary = transposed_location_summary
        transposed_location_summary.to_csv("well_location_summary.csv", index=False)


# experiment = LibraryExperiment(8,30,10,"gb_ref/UTI89.gb", "bowtie_ref/UTI89", 
#                                 "AGATGTGTATAAGAGACAG", 1, "input", "barcodes.csv",
#                                 True, "CGAGGTCTCT", "CGTACGCTGC")
# experiment.get_genomic_seq()        
# experiment.align_genomic_seq()        
# experiment.write_count_matrix()
# experiment.deconvolve()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pysudoku_package']

package_data = \
{'': ['*']}

install_requires = \
['biopython>=1.79',
 'click>=8.1.3',
 'numba>=0.55.2',
 'numpy>=1.22.4',
 'pandas>=1.4.2',
 'pytest',
 'scikit-learn>=1.1.1',
 'scipy>=1.8.1']

entry_points = \
{'console_scripts': ['arraylib-deconvolve = pysudoku_package.main:deconvolve',
                     'arraylib-run = pysudoku_package.main:run']}

setup_kwargs = {
    'name': 'arraylib-solve',
    'version': '0.5.0',
    'description': 'Tool to computationally deconvolve combinatorially pooled arrayed random mutagenesis libraries',
    'long_description': '# arraylib-solve\n\n# Introduction\n\n`arraylib-solve` is a tool to deconvolve combinatorially pooled arrayed random mutagenesis libraries (e.g. by transposon mutagenesis). In a typical experiment generating arrayed mutagenesis libraries, first a pooled version of the library is created and arrayed on a grid of well plates. To infer the identities of each mutant on the well plate, wells are pooled in combinatorial manner such that each mutant appears in a unique combination of pools. The pools are then sequenced using NGS and sequenced reads are stored in individual fastq files per pool. `arraylib-solve` deconvolves the pools and returns summaries stating the identity and location of each mutant on the original well grid. The package is based on the approach described in [[1]](#1).\n\n# Installation\n\nTo install `arraylib-solve` first create `Python 3.8` environment e.g. by\n\n```\nconda create --name arraylib-env python=3.8\nconda activate arraylib-env\n```\n\nand install the package using \n\n```\npip install arraylib-solve\n```\n\n`arraylib-solve` uses bowtie2 to aligne reads to the reference genome. Please ensure that bowtie2 is installed in your environment by running:\n\n```\nconda install bowtie2\n```\n\n\n# How to run `arraylib-solve`\n\nTo run `arraylib-solve` on a library deconvolution experiment with default parameters run:\n\n```\narraylib-run <input_directory> <experimental_design.csv> -c <number_of_cpu_cores_to_use> -b <path_to_genbank_reference> -br <path_to_bowtie2_indices> -t <transposon_sequence> -bu <upstream_sequence_of_barcodes> -bd <downstream_sequence_of_barcodes>\n```\n\n## Input parameters\n\nRequired parameters:\n* input_dir: path to directory holding the input fastq files\n* exp_design: path to file indicating experimental design. The experimental design file \n       should have columns, Filename, Poolname and Pooldimension. (see example in tests/test_data/full_exp_design.csv)\n       - Filename should contain all the unqiue input fastq filenames.\n       - Poolname should indicate to which pool a given file belongs. Multiple files per poolname are allowed.\n       - Pooldimension indicates the pooling dimension a pool belongs to. All pools sharing the same pooling dimension should have the same string in the Pooldimension column. \n* -gb path to genbank reference file\n* -br path to bowtie index files, ending with the basename of your index (if the basename of your index is UTI89 and you store your bowtie2 references in bowtie_ref it should be bowtie_ref/UTI89). Please visit https://bowtie-bio.sourceforge.net/bowtie2/manual.shtml#the-bowtie2-build-indexer for a manual how to create bowtie2 indices.\n* -t transposon sequence (e.g. ATTGCCTA)\n* -bu upstream sequence of barcode\n* -bd downstream sequence of barcode\n\nOptional parameters:\n* -mq minimum bowtie2 alignment quality score for each base to include read\n* -sq minimum phred score for each base to include read\n* -tm number of transposon mismatches allowed\n* -thr threshold for local filter (e.g. a threshold of 0.05 would filter out all reads < 0.05 of the maximum read count for a given mutant)\n\n## Output\n\n`arraylib-solve` outputs 4 files: \n* count_matrix.csv: Read counts per pool for each mutant.\n* filtered_matrix.csv: Read counts per pool for each mutant, but mutants with barcodes with low read counts for a given genomic location are filtered out.\n* mutant_location_summary.csv: A summary of mutants found in the well plate grid, where each row corresponds to a different mutant.\n* well_location_summary.csv: A summary of the deconvolved well plate grid, where each row corresponds to a different well.\n\n\n\n# References\n<a id="1">[1]</a> \nBaym, M., Shaket, L., Anzai, I.A., Adesina, O. and Barstow, B., 2016. Rapid construction of a whole-genome transposon insertion collection for Shewanella oneidensis by Knockout Sudoku. Nature communications, 7(1), p.13270.\n\n',
    'author': 'capraz',
    'author_email': 'tuemayc@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
